import {
  Metadata, IEventParam, Loader, IDatasource, IDictionary, I18n, Http,
} from '@zeedhi/core';
import {
  Form, Grid, GridEditable, Select, ModalService, Modal, Button, LoadingService, DialogService,
} from '@zeedhi/common';

export class Pme02500ValoresReferenciaDiario {
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public utilities = Loader.getInstance('Utilities');
  
  private showDetailValRef = false;
  
  public form?: Form;
  
  private modalAdd!: Modal;
  
  private modalExport!: Modal;
  
  public crudController = Loader.getInstance('CrudController');
  
  public get valRefIsVisible() {
    return !this.showDetailValRef;
  }
  
  private valRefCode = '';
  
  private rowClickedValue?: IDictionary;
  
  private modalAff!: Modal;
  
  public async valRefRowClick({ row }: IEventParam<Grid>) {
    const grid = Metadata.getInstance<GridEditable>('VALREFDIARIO_GRID');
    if (!grid.editing && !this.gridEditController.newRow && !this.gridEditController.isInlineEditing) {
      this.showDetailValRef = true;
      this.valRefCode = row.NRVALREFDIARIO;
  
      this.form!.value = row;
      this.rowClickedValue = row;
      
      const micro = Metadata.getInstance<GridEditable>('MICRONUTRIENTES');
      const formFilter = Metadata.getInstance<Form>('formFilter');
      micro.datasource.filter = [
        {
          name: 'NRVALREFDIARIO',
          operator: '=',
          value: this.form!.value.NRVALREFDIARIO,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: formFilter.value.CDTIPOCARD.CDTIPOCARD,
        },
      ];
      micro.datasource.get();
    }
  }
  
  public showValRefGrid() {
    this.form!.clearForm();
    this.form!.resetValidation();
    this.cancelFormMacro();
    const microGrid = Metadata.getInstance<GridEditable>('MICRONUTRIENTES');
    if (microGrid.editing) this.gridEditController.defaultState(microGrid);
    this.showDetailValRef = false;
    this.valRefCode = '';
  }
  
  public setFilterValRefDiario({ component }: IEventParam<Select>) {
    const grid = Metadata.getInstance<GridEditable>('VALREFDIARIO_GRID');
    grid.datasource.filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: component.value.CDTIPOCARD,
      },
    ];
    grid.datasource.get();
  }
  
  public loadFormInstance() {
    this.form = Metadata.getInstance<Form>('formValRefDiario');
  }
  
  public saveAddValRef() {
    const form = Metadata.getInstance<Form>('addValRef_containerModalForm');
    if (form.validate() && this.validateMinAndMaxValues(form.value)) {
      const row = form.value;
      const grid = Metadata.getInstance<GridEditable>('VALREFDIARIO_GRID');
      const ds = (grid.datasource as IDatasource);
      row.__is_new = true;
      row.NRVALREFDIARIO = ds.total + 1;
      row.CDFILCENTPME = this.appController.getCdCentral();
      const formFilter = Metadata.getInstance<Form>('formFilter');
      row.CDTIPOCARD = formFilter.value.CDTIPOCARD.CDTIPOCARD;
      const datasourceName = 'valRefDiario';
      LoadingService.show(' ');
      this.crudController.saveRows([row], datasourceName, null).then(() => {
        this.closeModalAddValRef();
        if (!grid.editing) grid.datasource.get();
        else this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }
  
  private validateMinAndMaxValues(formValue: IDictionary) {
    const result = Object.keys(formValue).some((key: any) => {
      let r: boolean;
      if (key.indexOf('MIN') > -1) {
        const keyMax = key.replace('MIN', 'MAX');
        r = (formValue[key] > formValue[keyMax]);
      } else if (key.indexOf('MAX') > -1) {
        const keyMin = key.replace('MAX', 'MIN');
        r = (formValue[key] < formValue[keyMin]);
      } else r = false;
      return r;
    });
    if (result) this.utilities.showAlert('alert', 'Verifique os valores mínimos e máximos informados!', 'error');
    return !result;
  }
  
  public async createModalAddValRef() {
    this.modalAdd = await ModalService.createFromJson(
      '/pme/parametrizacao/modals/add_pme02500_valores_ref_diario', true,
    );
    this.modalAdd.escKeydownStop = false;
  }
  
  public closeModalAddValRef() {
    this.modalAdd.hide();
    this.modalAdd.destroy();
    this.modalAdd = this.utilities.modalEmpty;
  }
    
  public async openModalAddValRef() {
    const form = Metadata.getInstance<Form>('formFilter');
    if (form.validate()) {
      if (!this.modalAdd) {
        this.createModalAddValRef().then(() => {
          this.modalAdd.show();
        });
      } else {
        await this.modalAdd.show();
      }
    }
  }
  
  public saveMicro() {
    const grid = Metadata.getInstance<GridEditable>('MICRONUTRIENTES');
    if (grid.isGridValid()) {
      const rows = grid.getEditedRows();
      const ds = (grid.datasource as IDatasource);
      let lastCode = ds.total + 1;
      const formFilter = Metadata.getInstance<Form>('formFilter');
      rows.forEach((el: any) => {
        if (el.__is_new) {
          el.NRVALREFDIARIO = this.valRefCode;
          el.NRREFDIARIOMICRO = lastCode;
          el.CDFILCENTPME = this.appController.getCdCentral();
          el.CDTIPOCARD = formFilter.value.CDTIPOCARD.CDTIPOCARD;
          lastCode += 1;
        }
      });
      const datasourceName = 'valRefDiarioMicroNutri';
      LoadingService.show(' ');
      this.crudController.saveRows(rows, datasourceName, grid).then(() => {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }
  
  public disableButtonsFormMacro() {
    const cancel = Metadata.getInstance<Button>('cancelButtonFormMacro');
    const save = Metadata.getInstance<Button>('saveButtonFormMacro');
    cancel.disabled = false;
    save.disabled = false;
  }
  
  public cancelFormMacro() {
    // this.mountedDadosPlate();
    const form = Metadata.getInstance<Form>('formValRefDiario');
    form.value = this.rowClickedValue as IDictionary;
    this.form!.value = this.rowClickedValue as IDictionary;
    
    const cancel = Metadata.getInstance<Button>('cancelButtonFormMacro');
    const save = Metadata.getInstance<Button>('saveButtonFormMacro');
    cancel.disabled = true;
    save.disabled = true;
  }
  
  public saveFormMacro() {
    const form = Metadata.getInstance<Form>('formValRefDiario');
    if (form.validate() && this.validateMinAndMaxValues(form.value)) {
      const rows = [form.value];
      const datasourceName = 'valRefDiario';
      LoadingService.show(' ');
      this.crudController.saveRows(rows, datasourceName, null).then(() => {
        this.rowClickedValue = form.value;
        this.form!.value = this.rowClickedValue as IDictionary;
        
        const cancel = Metadata.getInstance<Button>('cancelButtonFormMacro');
        const save = Metadata.getInstance<Button>('saveButtonFormMacro');
        cancel.disabled = true;
        save.disabled = true;
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }
  
  public clickExport() {
    const form = Metadata.getInstance<Form>('formFilter');
    const grid = Metadata.getInstance<GridEditable>('VALREFDIARIO_GRID');
    if (!form.validate() || grid.selectedRows.length === 0) {
      this.utilities.showAlert('alert', 'Selecione um registro para exportar.', 'error');
    } else {
      this.openModalExportValRef();
    }
  }
  
  public async createModalExportValRef() {
    this.modalExport = await ModalService.createFromJson(
      '/pme/parametrizacao/modals/pme02502_export_valores_ref_diario', true,
    );
    this.modalExport.escKeydownStop = false;
  }
  
  public closeModalExportValRef() {
    this.modalExport.hide();
    this.modalExport.destroy();
    this.modalExport = this.utilities.modalEmpty;
  }
    
  public async openModalExportValRef() {
    if (!this.modalExport) {
      this.createModalExportValRef().then(() => {
        this.modalExport.show();
      });
    } else {
      await this.modalExport.show();
    }
  }
  
  public clearDependentFieldExport() {
    const tpCard: Select = Metadata.getInstance('TIPOCARD');
    tpCard.setValue(null);
  }
  
  public setFilterTpCardapioExport({ component }: IEventParam<Select>) {
    const central = Metadata.getInstance<Select>('CENTRAL');
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: central.value,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public confirmExportValRef() {
    const form = Metadata.getInstance<Form>('formExportValRef');
    if (form.validate()) {
      DialogService.show({
        name: 'executeExport',
        type: 'warning',
        title: 'Atenção',
        text: 'Os dados de destino serão sobrespostos pelos de origem, deseja executar a exportação?',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: { click: () => this.exportValRef() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public exportValRef() {
    DialogService.hide();
    LoadingService.show(' ');
    const grid = Metadata.getInstance<GridEditable>('VALREFDIARIO_GRID');
    const form = Metadata.getInstance<Form>('formExportValRef');
    const formFilter = Metadata.getInstance<Form>('formFilter');
    const row = {
      CDFILCENTPME: form.value.CENTRAL,
      CDTIPOCARD: form.value.TIPOCARD.length > 0 ? form.value.TIPOCARD.join('_') : 'T',
      selectedRows: grid.selectedRows,
      CDFILCENTPME_ORIG: this.appController.getCdCentral(),
      CDTIPOCARD_ORIG: formFilter.value.CDTIPOCARD.CDTIPOCARD,
    };
    Http.post('/exportValRef', { row }).then((response: IDictionary) => {
      if (response.data.length === 0) {
        this.utilities.showAlert('alertSuccess', 'Exportação realizada com sucesso!', 'success');
        this.closeModalExportValRef();
        this.gridEditController.cancelEditing({ component: grid });
      } else {
        const mensagem = response.data.dataset.message.message;
        this.utilities.showAlert('alertError', I18n.translate(mensagem), 'error');
      }
    }).catch((error: IDictionary) => {
      this.utilities.showAlert('alertError', error, 'error');
    }).finally(() => {
      LoadingService.hide();
    });
  }
  
  public getMinFieldForm() {
    return 1;
  }
}
